==> fap-0.9.2.tar.bz2: fap-0.9.2/src/gui/list.cpp <==
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "list.h"
#include "utils.h"

#define ICON_SPACING             4    // Spacing between icon and label
#define SIDE_SPACING             6    // Left or right spacing between items

FXIMPLEMENT(listItem, FXListItem, 0, 0)

void listItem::draw(const FXList* list, FXDC& dc, FXint x, FXint y, FXint h) const {

	x+=SIDE_SPACING/2;
	if (icon) {
		dc.drawIcon(icon, x, y+(h-icon->getHeight())/2);
		x+=ICON_SPACING+icon->getWidth();
	}

	if (!_text.empty()) {
		register FXFont *font=list->getFont();
		dc.setForeground((!isEnabled())?makeShadowColor(list->getBackColor()):
			isSelected()?list->getSelTextColor():list->getTextColor());
		dc.drawText(x, y+(h-font->getFontHeight())/2+font->getFontAscent(), _text);
	}
}

FXint listItem::getWidth(const FXList* list) const {

	register FXint w=SIDE_SPACING;
	if (icon)
		w+=icon->getWidth()+ICON_SPACING;

	if (!_text.empty())
		w+=list->getFont()->getTextWidth(_text);

	return w;
}

void listItem::setText(const FXString& txt) {

	label=txt;
	_text=transform(txt);
}

/*****************************************************************/

FXDEFMAP(list) listMap[] = {
	FXMAPFUNC(SEL_PAINT,	0,	list::onPaint),
};

FXIMPLEMENT(list, FXList, listMap, ARRAYNUMBER(listMap))

list::list(FXComposite* p, FXObject* tgt, FXSelector sel, FXuint opts) :
	FXList(p, tgt, sel, opts),
	altbackColor(altColor(getBackColor())) {}

listItem* list::createItem(const FXString& text, FXIcon* icon, void* ptr) {

	return new listItem(text, icon, ptr);
}

long list::onPaint(FXObject*, FXSelector, void* ptr) {

	FXEvent* event=(FXEvent*)ptr;
	FXDCWindow dc(this, event);

	dc.setFont(font);

	// Paint items
	FXint y=pos_y;
	FXint w=FXMAX(listWidth, viewport_w);
	for (int i=0; i<items.no(); i++) {
		FXint h=items[i]->getHeight(this);
		if (event->rect.y<=y+h && y<event->rect.y+event->rect.h) {
			FXColor col=items[i]->isSelected()?selbackColor:(i&1)?altbackColor:backColor;
			dc.setForeground(col);
			dc.fillRectangle(pos_x, y, w, h-1);
			dc.setForeground(makeShadowColor(col));
			dc.fillRectangle(pos_x, y+h-1, w, 1);
			((listItem*)items[i])->draw(this, dc, pos_x, y, h);
		}
		y+=h;
	}

	// Paint blank area below items
	if (y<event->rect.y+event->rect.h) {
		dc.setStipple(STIPPLE_GRAY);
		dc.setFillStyle(FILL_OPAQUESTIPPLED);
		dc.setForeground(altbackColor);
		dc.setBackground(backColor);
		dc.fillRectangle(event->rect.x, y, event->rect.w, event->rect.y+event->rect.h-y);
	}
	return 1;
}

void list::setAltBackColor(FXColor clr) {

	if (altbackColor!=clr) {
		altbackColor=clr;
		update();
	}
}

==> fap-0.11.1.tar.bz2: fap-0.11.1/src/gui/list.cpp <==
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "list.h"
#include "utils.h"

#define ICON_SPACING             4    // Spacing between icon and label
#define SIDE_SPACING             6    // Left or right spacing between items

FXIMPLEMENT(listItem, FXListItem, 0, 0)

void listItem::draw(const FXList* list, FXDC& dc, FXint x, FXint y, FXint h) {

	x+=SIDE_SPACING/2;
	if (icon) {
		dc.drawIcon(icon, x, y+(h-icon->getHeight())/2);
		x+=ICON_SPACING+icon->getWidth();
	}

	if (!_text.empty()) {
		register FXFont *font=list->getFont();
		dc.setForeground((!isEnabled())?makeShadowColor(list->getBackColor()):
			isSelected()?list->getSelTextColor():list->getTextColor());
		dc.drawText(x, y+(h-font->getFontHeight())/2+font->getFontAscent(), _text);
	}
}

FXint listItem::getWidth(const FXList* list) const {

	register FXint w=SIDE_SPACING;
	if (icon)
		w+=icon->getWidth()+ICON_SPACING;

	if (!_text.empty())
		w+=list->getFont()->getTextWidth(_text);

	return w;
}

void listItem::setText(const FXString& txt) {

	label=txt;
	_text=transform(txt);
}

/*****************************************************************/

FXDEFMAP(list) listMap[] = {
	FXMAPFUNC(SEL_PAINT,	0,	list::onPaint),
};

FXIMPLEMENT(list, FXList, listMap, ARRAYNUMBER(listMap))

list::list(FXComposite* p, FXObject* tgt, FXSelector sel, FXuint opts) :
	FXList(p, tgt, sel, opts),
	altbackColor(altColor(getBackColor())) {}

listItem* list::createItem(const FXString& text, FXIcon* icon, void* ptr) {

	return new listItem(text, icon, ptr);
}

long list::onPaint(FXObject*, FXSelector, void* ptr) {

	FXEvent* event=(FXEvent*)ptr;
	FXDCWindow dc(this, event);

	dc.setFont(font);

	// Paint items
	FXint y=pos_y;
	FXint w=FXMAX(listWidth, viewport_w);
	for (int i=0; i<items.no(); i++) {
		FXint h=items[i]->getHeight(this);
		if (event->rect.y<=y+h && y<event->rect.y+event->rect.h) {
			FXColor col=items[i]->isSelected()?selbackColor:(i&1)?altbackColor:backColor;
			dc.setForeground(col);
			dc.fillRectangle(pos_x, y, w, h-1);
			dc.setForeground(makeShadowColor(col));
			dc.fillRectangle(pos_x, y+h-1, w, 1);
			((listItem*)items[i])->draw(this, dc, pos_x, y, h);
		}
		y+=h;
	}

	// Paint blank area below items
	if (y<event->rect.y+event->rect.h) {
		dc.setStipple(STIPPLE_GRAY);
		dc.setFillStyle(FILL_OPAQUESTIPPLED);
		dc.setForeground(altbackColor);
		dc.setBackground(backColor);
		dc.fillRectangle(event->rect.x, y, event->rect.w, event->rect.y+event->rect.h-y);
	}
	return 1;
}

void list::setAltBackColor(FXColor clr) {

	if (altbackColor!=clr) {
		altbackColor=clr;
		update();
	}
}
